@@ -8,9 +8,10 @@
 
 #include "nsIAccessiblePivot.h"
 
-#include "AccEvent.h"
 #include "HyperTextAccessibleWrap.h"
+#include "AccEvent.h"
 
+#include "nsAutoPtr.h"
 #include "nsClassHashtable.h"
 #include "nsDataHashtable.h"
 #include "nsIDocument.h"
@@ -50,8 +51,7 @@ class DocAccessible : public HyperTextAccessibleWrap,
 
 public:
 
-  DocAccessible(nsIDocument* aDocument, nsIContent* aRootContent,
-                nsIPresShell* aPresShell);
+  DocAccessible(nsIDocument* aDocument, nsIPresShell* aPresShell);
 
   // nsIScrollPositionListener
   virtual void ScrollPositionWillChange(nscoord aX, nscoord aY) override {}
@@ -188,6 +188,7 @@ public:
    */
   void FireDelayedEvent(AccEvent* aEvent);
   void FireDelayedEvent(uint32_t aEventType, Accessible* aTarget);
+  void FireEventsOnInsertion(Accessible* aContainer);
 
   /**
    * Fire value change event on the given accessible if applicable.
@@ -226,7 +227,11 @@ public:
    *
    * @return the accessible object
    */
-  Accessible* GetAccessible(nsINode* aNode) const;
+  Accessible* GetAccessible(nsINode* aNode) const
+  {
+    return aNode == mDocumentNode ?
+      const_cast<DocAccessible*>(this) : mNodeToAccessibleMap.Get(aNode);
+  }
 
   /**
    * Return an accessible for the given node even if the node is not in
@@ -276,6 +281,12 @@ public:
     return aNode ? GetAccessibleOrContainer(aNode->GetParentNode()) : nullptr;
   }
 
+  /**
+   * Return an accessible for the given node if any, or an immediate accessible
+   * container for it.
+   */
+  Accessible* AccessibleOrTrueContainer(nsINode* aNode) const;
+
   /**
    * Return an accessible for the given node or its first accessible descendant.
    */
@@ -292,6 +303,11 @@ public:
     }
     return nullptr;
   }
+  uint32_t ARIAOwnedCount(Accessible* aParent) const
+  {
+    nsTArray<RefPtr<Accessible> >* children = mARIAOwnsHash.Get(aParent);
+    return children ? children->Length() : 0;
+  }
 
   /**
    * Return true if the given ID is referred by relation attribute.
@@ -310,7 +326,8 @@ public:
    * @param  aRoleMapEntry  [in] the role map entry role the ARIA role or nullptr
    *                          if none
    */
-  void BindToDocument(Accessible* aAccessible, nsRoleMapEntry* aRoleMapEntry);
+  void BindToDocument(Accessible* aAccessible,
+                      const nsRoleMapEntry* aRoleMapEntry);
 
   /**
    * Remove from document and shutdown the given accessible.
@@ -348,6 +365,17 @@ public:
    */
   void RecreateAccessible(nsIContent* aContent);
 
+  /**
+   * Schedule ARIA owned element relocation if needed. Return true if relocation
+   * was scheduled.
+   */
+  bool RelocateARIAOwnedIfNeeded(nsIContent* aEl);
+
+  /**
+   * Return a notification controller associated with the document.
+   */
+  NotificationController* Controller() const { return mNotificationController; }
+
   /**
    * If this document is in a content process return the object responsible for
    * communicating with the main process for it.
@@ -359,9 +387,6 @@ protected:
 
   void LastRelease();
 
-  // Accessible
-  virtual void CacheChildren() override;
-
   // DocAccessible
   virtual nsresult AddEventListeners();
   virtual nsresult RemoveEventListeners();
@@ -380,6 +405,11 @@ protected:
    */
   virtual void DoInitialUpdate();
 
+  /**
+   * Updates root element and picks up ARIA role on it if any.
+   */
+  void UpdateRootElIfNeeded();
+
   /**
    * Process document load notification, fire document load and state busy
    * events if applicable.
@@ -470,6 +500,8 @@ protected:
    */
   void ProcessContentInserted(Accessible* aContainer,
                               const nsTArray<nsCOMPtr<nsIContent> >* aInsertedContent);
+  void ProcessContentInserted(Accessible* aContainer,
+                              nsIContent* aInsertedContent);
 
   /**
    * Used to notify the document to make it process the invalidation list.
@@ -480,34 +512,11 @@ protected:
    */
   void ProcessInvalidationList();
 
-  /**
-   * Update the tree on content insertion.
-   */
-  void UpdateTreeOnInsertion(Accessible* aContainer);
-
   /**
    * Update the accessible tree for content removal.
    */
   void UpdateTreeOnRemoval(Accessible* aContainer, nsIContent* aChildNode);
 
-  /**
-   * Helper for UpdateTreeOn methods. Go down to DOM subtree and updates
-   * accessible tree. Return one of these flags.
-   */
-  enum EUpdateTreeFlags {
-    eNoAccessible = 0,
-    eAccessible = 1,
-    eAlertAccessible = 2
-  };
-
-  uint32_t UpdateTreeInternal(Accessible* aChild, bool aIsInsert,
-                              AccReorderEvent* aReorderEvent);
-
-  /**
-   * Schedule ARIA owned element relocation if needed.
-   */
-  void RelocateARIAOwnedIfNeeded(nsIContent* aEl);
-
   /**
    * Validates all aria-owns connections and updates the tree accordingly.
    */
@@ -518,23 +527,15 @@ protected:
    */
   void DoARIAOwnsRelocation(Accessible* aOwner);
 
-  /**
-   * Moves the child from old parent under new one.
-   */
-  bool SeizeChild(Accessible* aNewParent, Accessible* aChild,
-                  int32_t aIdxInParent);
-
-  /**
-   * Move the child under same parent.
-   */
-  void MoveChild(Accessible* aChild, int32_t aIdxInParent);
-
   /**
    * Moves children back under their original parents.
    */
   void PutChildrenBack(nsTArray<RefPtr<Accessible> >* aChildren,
                        uint32_t aStartIdx);
 
+  bool MoveChild(Accessible* aChild, Accessible* aNewParent,
+                 int32_t aIdxInParent);
+
   /**
    * Create accessible tree.
    *
@@ -544,6 +545,7 @@ protected:
    */
   void CacheChildrenInSubtree(Accessible* aRoot,
                               Accessible** aFocusedAcc = nullptr);
+  void CreateSubtree(Accessible* aRoot);
 
   /**
    * Remove accessibles in subtree from node to accessible map.
@@ -575,7 +577,7 @@ protected:
    */
   void SetIPCDoc(DocAccessibleChild* aIPCDoc) { mIPCDoc = aIPCDoc; }
 
-  friend class DocAccessibleChild;
+  friend class DocAccessibleChildBase;
 
   /**
    * Used to fire scrolling end event after page scroll.
@@ -693,7 +695,7 @@ protected:
    * Used to process notification from core and accessible events.
    */
   RefPtr<NotificationController> mNotificationController;
-  friend class EventQueue;
+  friend class EventTree;
   friend class NotificationController;
 
 private:
