@@ -24,9 +24,7 @@ ia2AccessibleEditableText::copyText(long aStartOffset, long aEndOffset)
 {
   A11Y_TRYBLOCK_BEGIN
 
-  if (ProxyAccessible* proxy = HyperTextProxyFor(this)) {
-    return proxy->CopyText(aStartOffset, aEndOffset) ? S_OK : E_INVALIDARG;
-  }
+  MOZ_ASSERT(!HyperTextProxyFor(this));
 
   HyperTextAccessible* textAcc = static_cast<HyperTextAccessibleWrap*>(this);
   if (textAcc->IsDefunct())
@@ -46,9 +44,7 @@ ia2AccessibleEditableText::deleteText(long aStartOffset, long aEndOffset)
 {
   A11Y_TRYBLOCK_BEGIN
 
-    if (ProxyAccessible* proxy = HyperTextProxyFor(this)) {
-      return proxy->DeleteText(aStartOffset, aEndOffset) ? S_OK : E_INVALIDARG;
-    }
+  MOZ_ASSERT(!HyperTextProxyFor(this));
 
   HyperTextAccessible* textAcc = static_cast<HyperTextAccessibleWrap*>(this);
   if (textAcc->IsDefunct())
@@ -70,9 +66,7 @@ ia2AccessibleEditableText::insertText(long aOffset, BSTR *aText)
 
   uint32_t length = ::SysStringLen(*aText);
   nsAutoString text(*aText, length);
-  if (ProxyAccessible* proxy = HyperTextProxyFor(this)) {
-    return proxy->InsertText(text, aOffset) ? S_OK : E_INVALIDARG;
-  }
+  MOZ_ASSERT(!HyperTextProxyFor(this));
 
   HyperTextAccessible* textAcc = static_cast<HyperTextAccessibleWrap*>(this);
   if (textAcc->IsDefunct())
@@ -92,9 +86,7 @@ ia2AccessibleEditableText::cutText(long aStartOffset, long aEndOffset)
 {
   A11Y_TRYBLOCK_BEGIN
 
-  if (ProxyAccessible* proxy = HyperTextProxyFor(this)) {
-    return proxy->CutText(aStartOffset, aEndOffset) ? S_OK : E_INVALIDARG;
-  }
+  MOZ_ASSERT(!HyperTextProxyFor(this));
 
   HyperTextAccessible* textAcc = static_cast<HyperTextAccessibleWrap*>(this);
   if (textAcc->IsDefunct())
@@ -114,9 +106,7 @@ ia2AccessibleEditableText::pasteText(long aOffset)
 {
   A11Y_TRYBLOCK_BEGIN
 
-  if (ProxyAccessible* proxy = HyperTextProxyFor(this)) {
-    return proxy->PasteText(aOffset) ? S_OK : E_INVALIDARG;
-  }
+  MOZ_ASSERT(!HyperTextProxyFor(this));
 
   HyperTextAccessible* textAcc = static_cast<HyperTextAccessibleWrap*>(this);
   if (textAcc->IsDefunct())
