@@ -90,7 +90,7 @@ function testSteps()
       req.result.continue();
     }
     e = yield undefined;
-    is(req.result, undefined, "exhausted indexes");
+    ok(req.result == null, "exhausted indexes");
 
     let tempIndex = store.createIndex("temp index", "x", { multiEntry: true });
     req = tempIndex.openKeyCursor();
@@ -102,7 +102,7 @@ function testSteps()
       req.result.continue();
     }
     e = yield undefined;
-    is(req.result, undefined, "exhausted temp index");
+    ok(req.result == null, "exhausted temp index");
     store.deleteIndex("temp index");
   }
 
@@ -186,7 +186,7 @@ function testSteps()
       req.result.continue();
     }
     e = yield undefined;
-    is(req.result, undefined, "exhausted indexes");
+    ok(req.result == null, "exhausted indexes");
 
     let tempIndex = store.createIndex("temp index", "x", { multiEntry: true, unique: true });
     req = tempIndex.openKeyCursor();
@@ -198,7 +198,7 @@ function testSteps()
       req.result.continue();
     }
     e = yield undefined;
-    is(req.result, undefined, "exhausted temp index");
+    ok(req.result == null, "exhausted temp index");
     store.deleteIndex("temp index");
   }
 
