@@ -8,6 +8,7 @@
 #include "mozilla/dom/HTMLMediaElement.h"
 #include "mozilla/Services.h"
 #include "AndroidMediaPluginHost.h"
+#include "nsAutoPtr.h"
 #include "nsXPCOMStrings.h"
 #include "nsISeekableStream.h"
 #include "nsIGfxInfo.h"
@@ -42,11 +43,11 @@ static char* GetResource(Decoder *aDecoder)
   return static_cast<char*>(aDecoder->mResource);
 }
 
-class GetIntPrefEvent : public nsRunnable {
+class GetIntPrefEvent : public Runnable {
 public:
   GetIntPrefEvent(const char* aPref, int32_t* aResult)
     : mPref(aPref), mResult(aResult) {}
-  NS_IMETHOD Run() {
+  NS_IMETHOD Run() override {
     return Preferences::GetInt(mPref, mResult);
   }
 private:
@@ -114,7 +115,8 @@ static bool IsOmxSupported()
     nsCOMPtr<nsIGfxInfo> gfxInfo = services::GetGfxInfo();
     if (gfxInfo) {
       int32_t status;
-      if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_STAGEFRIGHT, &status))) {
+      nsCString discardFailure;
+      if (NS_SUCCEEDED(gfxInfo->GetFeatureStatus(nsIGfxInfo::FEATURE_STAGEFRIGHT, discardFailure, &status))) {
         if (status != nsIGfxInfo::FEATURE_STATUS_OK) {
           NS_WARNING("XXX stagefright blacklisted\n");
           return false;
@@ -174,29 +176,6 @@ static const char* GetOmxLibraryName()
   if (version >= 17) {
     return "libomxpluginkk.so";
   }
-  else if (version == 13 || version == 12 || version == 11) {
-    return "libomxpluginhc.so";
-  }
-  else if (version == 10 && release_version >= NS_LITERAL_STRING("2.3.6")) {
-    // Gingerbread versions from 2.3.6 and above have a different DataSource
-    // layout to those on 2.3.5 and below.
-    return "libomxplugingb.so";
-  }
-  else if (version == 10 && release_version >= NS_LITERAL_STRING("2.3.4") &&
-           device.Find("HTC") == 0) {
-    // HTC devices running Gingerbread 2.3.4+ (HTC Desire HD, HTC Evo Design, etc) seem to
-    // use a newer version of Gingerbread libstagefright than other 2.3.4 devices.
-    return "libomxplugingb.so";
-  }
-  else if (version == 9 || (version == 10 && release_version <= NS_LITERAL_STRING("2.3.5"))) {
-    // Gingerbread versions from 2.3.5 and below have a different DataSource
-    // than 2.3.6 and above.
-    return "libomxplugingb235.so";
-  }
-  else if (version < 9) {
-    // Below Gingerbread not supported
-    return nullptr;
-  }
 
   // Ice Cream Sandwich and Jellybean
   return "libomxplugin.so";
