@@ -10,6 +10,7 @@
 #include "WidevineFileIO.h"
 #include <mozilla/SizePrintfMacros.h>
 #include <stdarg.h>
+#include "base/time.h"
 
 using namespace cdm;
 using namespace std;
@@ -297,11 +298,7 @@ WidevineDecryptor::SetTimer(int64_t aDelayMs, void* aContext)
 Time
 WidevineDecryptor::GetCurrentWallTime()
 {
-  GMPTimestamp gmpTime = 0;
-  GMPGetCurrentTime(&gmpTime);
-  double t = (double)gmpTime / 1e3;
-  Log("Decryptor::GetCurrentWallTime()= %lf", t);
-  return t;
+  return base::Time::Now().ToDoubleT();
 }
 
 void
