@@ -1,7 +1,6 @@
 /*
  * This file is part of SpyBlock <https://www.gnu.org/software/gnuzilla/>,
- * Copyright (C) 2006-2015 Eyeo GmbH (Adblock Plus) 
- * Copyright (C) 2017 Ruben Rodriguez <ruben@gnu.org>
+ * Copyright (C) 2006-2017 eyeo GmbH
  *
  * SpyBlock is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 3 as
@@ -16,23 +15,28 @@
  * along with SpyBlock.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-Cu.import("resource://gre/modules/Services.jsm");
+"use strict";
 
-let {Utils} = require("utils");
-let {Prefs} = require("prefs");
-let {ActiveFilter} = require("filterClasses");
-let {FilterStorage} = require("filterStorage");
-let {FilterNotifier} = require("filterNotifier");
-let {Subscription} = require("subscriptionClasses");
-let {Notification} = require("notification");
+const {Prefs} = require("prefs");
+const {ActiveFilter} = require("filterClasses");
+const {FilterStorage} = require("filterStorage");
+const {FilterNotifier} = require("filterNotifier");
+const {Subscription} = require("subscriptionClasses");
+const {Notification} = require("notification");
+
+let ext;
+if (typeof window != "undefined" && window.ext)
+  ({ext} = window);
+else
+  ext = require("ext_background");
 
 exports.initAntiSpyBlockNotification = function initAntiSpyBlockNotification()
 {
   let notification = {
     id: "antiadblock",
     type: "question",
-    title: Utils.getString("notification_antiadblock_title"),
-    message: Utils.getString("notification_antiadblock_message"),
+    title: ext.i18n.getMessage("notification_antiadblock_title"),
+    message: ext.i18n.getMessage("notification_antiadblock_message"),
     urlFilters: []
   };
 
@@ -52,8 +56,9 @@ exports.initAntiSpyBlockNotification = function initAntiSpyBlockNotification()
       {
         for (let domain in filter.domains)
         {
-          let urlFilter = "||" + domain + "^";
-          if (domain && filter.domains[domain] && urlFilters.indexOf(urlFilter) == -1)
+          let urlFilter = "||" + domain + "^$document";
+          if (domain && filter.domains[domain] &&
+              urlFilters.indexOf(urlFilter) == -1)
             urlFilters.push(urlFilter);
         }
       }
@@ -69,18 +74,25 @@ exports.initAntiSpyBlockNotification = function initAntiSpyBlockNotification()
     Notification.removeQuestionListener(notification.id, notificationListener);
   }
 
-  let subscription = Subscription.fromURL(Prefs.subscriptions_antiadblockurl);
-  if (subscription.lastDownload && subscription.disabled)
-    addAntiSpyBlockNotification(subscription);
+  let antiSpyBlockSubscription = Subscription.fromURL(
+    Prefs.subscriptions_antiadblockurl
+  );
+  if (antiSpyBlockSubscription.lastDownload && antiSpyBlockSubscription.disabled)
+    addAntiSpyBlockNotification(antiSpyBlockSubscription);
 
-  FilterNotifier.addListener(function(action, value, newItem, oldItem)
+  function onSubscriptionChange(subscription)
   {
-    if (!/^subscription\.(updated|removed|disabled)$/.test(action) || value.url != Prefs.subscriptions_antiadblockurl)
+    let url = Prefs.subscriptions_antiadblockurl;
+    if (url != subscription.url)
       return;
 
-    if (action == "subscription.updated")
-      addAntiSpyBlockNotification(value);
-    else if (action == "subscription.removed" || (action == "subscription.disabled" && !value.disabled))
+    if (url in FilterStorage.knownSubscriptions && subscription.disabled)
+      addAntiSpyBlockNotification(subscription);
+    else
       removeAntiSpyBlockNotification();
-  });
-}
+  }
+
+  FilterNotifier.on("subscription.updated", onSubscriptionChange);
+  FilterNotifier.on("subscription.removed", onSubscriptionChange);
+  FilterNotifier.on("subscription.disabled", onSubscriptionChange);
+};
